from msorm import models
__preview__ = False
__MAJOR__ = 1
__MINOR__ = 1
__MAINTENANCE__ = 14
__preview_version__ = f"{__MAJOR__}.{__MINOR__}.{__MAINTENANCE__}-rc2"
__version__ = f"{__MAJOR__}.{__MINOR__}.{__MAINTENANCE__}-a0"
__version__ = __preview_version__ if __preview__ else __version__
__author__ = "Mehmet Berkay Özbay"
__author_git__ = "https://github.com/bilinenkisi"
__git__ = "https://github.com/bilinenkisi/msorm"
